<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

/**
 * Обрабатывает вывод отчетов профилирования
 *
 */
class PluginTeams_ActionTeams extends ActionPlugin {
	/**
	 * Текущий юзер
	 *
	 * @var UserEntity_User
	 */
	protected $oUserCurrent=null;
		
	/**
	 * Инициализация 
	 *
	 * @return null
	 */
	public function Init() {	
		/**
		 * Проверяем авторизован ли юзер
		 */
		$this->oUserCurrent=$this->User_GetUserCurrent();
		 
		$this->SetDefaultEvent('all');
	}
	
	protected function RegisterEvent() {		
		$this->AddEvent('all','EventTeams');
		$this->AddEvent('add','EventAddTeam');
		$this->AddEvent('edit','EventEditTeam');
		$this->AddEvent('delete','EventDeleteTeam');
		$this->AddEvent('admin','EventAdminTeam');
		$this->AddEvent('invite','EventInviteTeam');
		$this->AddEvent('ajaxaddteaminvite', 'AjaxAddTeamInvite');
		$this->AddEventPreg('/^[\w\-\_]+$/i','/^(page(\d+))?$/i','EventShowTeam');
	}
		
	
	/**********************************************************************************
	 ************************ РЕАЛИЗАЦИЯ ЭКШЕНА ***************************************
	 **********************************************************************************
	 */
	/**
	 * Добавление нового блога
	 *
	 * @return unknown
	 */
	 protected function EventTeams() {
		
		/**
		 * Получаем список блогов
		 */
		$aTeams=$this->PluginTeams_Teams_GetTeams();
		
		/**
		 * Загружаем переменные в шаблон
		 */		
		$this->Viewer_Assign('aTeams',$aTeams);	
		$this->Viewer_AddHtmlTitle($this->Lang_Get('teams'));
	}
	 
	 
	protected function EventAddTeam() {
		$this->Viewer_AddHtmlTitle($this->Lang_Get('team_create'));
		/**
		 * Меню
		 */
		$this->sMenuSubItemSelect='add';
		$this->sMenuItemSelect='add_team';				
		/**
		 * Проверяем авторизован ли пользователь
		 */
		if (!$this->User_IsAuthorization()) {
			$this->Message_AddErrorSingle($this->Lang_Get('not_access'),$this->Lang_Get('error'));
			return Router::Action('error');
		}		
		/**
		 * Проверяем хватает ли рейтинга юзеру чтоб создать блог
		 */
		if (!$this->oUserCurrent->isAdministrator()) {
			$this->Message_AddErrorSingle($this->Lang_Get('team_create_acl'),$this->Lang_Get('error'));
			return Router::Action('error');
		}		
		/**
		 * Запускаем проверку корректности ввода полей при добалении блога.
		 * Дополнительно проверяем, что был отправлен POST запрос.
		 */
		if (!$this->checkTeamFields()) {
			return false;	
		}	
		/**
		 * Если всё ок то пытаемся создать блог
		 */
		$oTeam=Engine::GetEntity('PluginTeams_Teams_Teams');
		$oTeam->setOwnerId($this->oUserCurrent->getId());
		$oTeam->setTitle(getRequest('team_title'));
		/**
		 * Парсим текст на предмет разных ХТМЛ тегов
		 */
		$sText=$this->Text_Parser(getRequest('team_description'));
		$oTeam->setDescription($sText);
		$oTeam->setDateAdd(date("Y-m-d H:i:s"));
		$oTeam->setUrl(getRequest('team_url'));
		$oTeam->setAvatar(null);
		/**
		* Загрузка аватара, делаем ресайзы
		*/			
		if (isset($_FILES['avatar']) and is_uploaded_file($_FILES['avatar']['tmp_name'])) {
			if ($sPath=$this->PluginTeams_Teams_UploadTeamAvatar($_FILES['avatar'],$oTeam)) {
				$oTeam->setAvatar($sPath);
			} else {
				$this->Message_AddError($this->Lang_Get('team_create_avatar_error'),$this->Lang_Get('error'));
				return false;
			}
		}
		/**
		 * Создаём блог
		 */
		$this->Hook_Run('team_add_before', array('oTeam'=>$oTeam));
		if ($this->PluginTeams_Teams_AddTeam($oTeam)) {
			$this->Hook_Run('team_add_after', array('oTeam'=>$oTeam));
			/**
			 * Получаем блог, это для получение полного пути блога, если он в будущем будет зависит от других сущностей(компании, юзер и т.п.)
			 */
			$oTeam->PluginTeams_Teams_GetTeamById($oTeam->getId());
			Router::Location($oTeam->getUrlFull());
		} else {
			$this->Message_AddError($this->Lang_Get('system_error'),$this->Lang_Get('error'));
		}
	}
	
	/**
	 * Редактирование блога
	 *
	 * @return unknown
	 */
	protected function EventEditTeam() {
		/**
		 * Меню
		 */
		$this->sMenuSubItemSelect='';
		$this->sMenuItemSelect='profile';
		
		/**
		 * Проверяем передан ли в УРЛе номер блога
		 */
		$sTeamId=$this->GetParam(0);
		if (!$oTeam=$this->PluginTeams_Teams_GetTeamById($sTeamId)) {
			return parent::EventNotFound();
		}
		/**
		 * Проверям авторизован ли пользователь
		 */
		if (!$this->User_IsAuthorization()) {
			$this->Message_AddErrorSingle($this->Lang_Get('not_access'),$this->Lang_Get('error'));
			return Router::Action('error');
		}
		/**
		 * Явлется ли авторизованный пользователь хозяином блога, либо его администратором
		 */
		$oTeamUser=$this->PluginTeams_Teams_GetTeamUserByTeamIdAndUserId($oTeam->getId(),$this->oUserCurrent->getId());		
		$bIsAdministratorTeam=$oTeamUser ? $oTeamUser->getIsAdministrator() : false;
		if ($oTeam->getOwnerId()!=$this->oUserCurrent->getId()  and !$this->oUserCurrent->isAdministrator() and !$bIsAdministratorTeam) {
			return parent::EventNotFound();
		}			
		
		$this->Viewer_AddHtmlTitle($oTeam->getTitle());
		$this->Viewer_AddHtmlTitle($this->Lang_Get('team_edit'));
		
		$this->Viewer_Assign('oTeamEdit',$oTeam);
		/**
		 * Устанавливаем шалон для вывода
		 */		
		$this->SetTemplateAction('add');
		/**
		 * Если нажали кнопку "Сохранить"
		 */
		if (isPost('submit_team_add')) {
			/**
			 * Запускаем проверку корректности ввода полей при редактировании блога
			 */
			if (!$this->checkTeamFields($oTeam)) {
				return false;
			}
			$oTeam->setTitle(getRequest('team_title'));
			/**
			 * Парсим описание блога на предмет ХТМЛ тегов
			 */
			$sText=$this->Text_Parser(getRequest('team_description'));			
			$oTeam->setDescription($sText);
			//$oBlog->setUrl(getRequest('blog_url'));	// запрещаем смену URL блога	
			/**
			* Загрузка аватара, делаем ресайзы
			*/			
			if (isset($_FILES['avatar']) and is_uploaded_file($_FILES['avatar']['tmp_name'])) {
				if ($sPath=$this->PluginTeams_Teams_UploadTeamAvatar($_FILES['avatar'],$oTeam)) {
					$oTeam->setAvatar($sPath);
				} else {
					$this->Message_AddError($this->Lang_Get('team_create_avatar_error'),$this->Lang_Get('error'));
					return false;
				}
			}
			/**
			 * Удалить аватар
			 */
			if (isset($_REQUEST['avatar_delete'])) {
				$this->PluginTeams_Teams_DeleteTeamAvatar($oTeam);
				$oTeam->setAvatar(null);
			}
			/**
			 * Обновляем блог
			 */
			$this->Hook_Run('team_edit_before', array('oTeam'=>$oTeam));
			if ($this->PluginTeams_Teams_UpdateTeam($oTeam)) {
				$this->Hook_Run('team_edit_after', array('oTeam'=>$oTeam));
				Router::Location($oTeam->getUrlFull());
			} else {
				$this->Message_AddErrorSingle($this->Lang_Get('system_error'),$this->Lang_Get('error'));
				return Router::Action('error');
			}
		} else {
			/**
			 * Загружаем данные в форму редактирования блога
			 */
			$_REQUEST['team_title']=$oTeam->getTitle();
			$_REQUEST['team_url']=$oTeam->getUrl();
			$_REQUEST['team_description']=$oTeam->getDescription();
			$_REQUEST['team_id']=$oTeam->getId();
		}
		
		
	}
	/**
	 * Управление пользователями блога
	 *
	 * @return unknown
	 */
	protected function EventAdminTeam() {		
		/**
		 * Меню
		 */		
		$this->sMenuItemSelect='admin';
		$this->sMenuSubItemSelect='';
		/**
		 * Проверяем передан ли в УРЛе номер блога
		 */
		$sTeamId=$this->GetParam(0);
		if (!$oTeam=$this->PluginTeams_Teams_GetTeamById($sTeamId)) {
			return parent::EventNotFound();
		}
		/**
		 * Проверям авторизован ли пользователь
		 */
		if (!$this->User_IsAuthorization()) {
			$this->Message_AddErrorSingle($this->Lang_Get('not_access'),$this->Lang_Get('error'));
			return Router::Action('error');
		}
		/**
		 * Явлется ли авторизованный пользователь хозяином блога, либо его администратором
		 */
		$oTeamUser=$this->PluginTeams_Teams_GetTeamUserByTeamIdAndUserId($oTeam->getId(),$this->oUserCurrent->getId());		
		$bIsAdministratorTeam=$oTeamUser ? $oTeamUser->getIsAdministrator() : false;
		if ($oTeam->getOwnerId()!=$this->oUserCurrent->getId()  and !$this->oUserCurrent->isAdministrator() and !$bIsAdministratorTeam) {
			return parent::EventNotFound();
		}					
		/**
		 * Обрабатываем сохранение формы
		 */
		if (isPost('submit_team_admin')) {
			$this->Security_ValidateSendForm();
			
			$aUserRank=getRequest('user_rank',array());
			if (!is_array($aUserRank)) {
				$aUserRank=array();
			}
			foreach ($aUserRank as $sUserId => $sRank) {
				if (!($oTeamUser=$this->PluginTeams_Teams_GetTeamUserByTeamIdAndUserId($oTeam->getId(),$sUserId))) {
					$this->Message_AddError($this->Lang_Get('system_error'),$this->Lang_Get('error'));
					break;
				}
				
				switch ($sRank) {
					case 'administrator':
						$oTeamUser->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_ADMINISTRATOR);
						break;
					case 'moderator':
						$oTeamUser->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_MODERATOR);
						break;
					case 'reader':
						$oTeamUser->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_USER);
						break;
					case 'ban':
						$oTeamUser->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_BAN);
						break;
					default:
						$oTeamUser->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_GUEST);						
				}
				$this->PluginTeams_Teams_UpdateRelationTeamUser($oTeamUser);
				$this->Message_AddNoticeSingle($this->Lang_Get('team_admin_users_submit_ok'));
			}
		}
		/**
		 * Получаем список подписчиков блога
		 */
		$aTeamUsers=$this->PluginTeams_Teams_GetTeamUsersByTeamId(
			$oTeam->getId(),
			array(
				PluginTeams_Teams::TEAM_USER_ROLE_BAN,
				PluginTeams_Teams::TEAM_USER_ROLE_USER,
				PluginTeams_Teams::TEAM_USER_ROLE_MODERATOR,
				PluginTeams_Teams::TEAM_USER_ROLE_ADMINISTRATOR
			)
		);

		$this->Viewer_AddHtmlTitle($oTeam->getTitle());
		$this->Viewer_AddHtmlTitle($this->Lang_Get('team_admin'));
		
		$this->Viewer_Assign('oTeamEdit',$oTeam);
		$this->Viewer_Assign('aTeamUsers',$aTeamUsers);
		/**
		 * Устанавливаем шалон для вывода
		 */		
		$this->SetTemplateAction('admin');
		
		/**
		 * Если блог закрытый, получаем приглашенных 
		 * и добавляем блок-форму для приглашения 
		 */
		$aTeamUsersInvited=$this->PluginTeams_Teams_GetTeamUsersByTeamId($oTeam->getId(),PluginTeams_Teams::TEAM_USER_ROLE_INVITE);				
			$this->Viewer_Assign('aTeamUsersInvited',$aTeamUsersInvited);
			$this->Viewer_AddBlock('right',$this->getTemplatePathPlugin().'/actions/ActionTeams/invited.tpl');
		
	}
	
	/**
	 * Проверка полей блога
	 *
	 * @return bool
	 */
	protected function checkTeamFields($oTeam=null) {
		/**
		 * Проверяем только если была отправлена форма с данными (методом POST)
		 */
		if (!isPost('submit_team_add')) {
			$_REQUEST['team_limit_rating_topic']=0;
			return false;
		}
		$this->Security_ValidateSendForm();
		
		$bOk=true;
		/**
		* Проверяем есть ли название блога
		*/
		if (!func_check(getRequest('team_title'),'text',2,200)) {
			$this->Message_AddError($this->Lang_Get('team_create_title_error'),$this->Lang_Get('error'));
			$bOk=false;
		}
		/**
		 * Проверяем есть ли уже блог с таким названием
		 */
		if ($oTeamExists=$this->PluginTeams_Teams_GetTeamByTitle(getRequest('team_title'))) {
			if (!$oTeam or $oTeam->getId()!=$oTeamExists->getId()) {
				$this->Message_AddError($this->Lang_Get('team_create_title_error_unique'),$this->Lang_Get('error'));
				$bOk=false;
			}
		}
		
		if (!$oTeam) {
			/**
			* Проверяем есть ли URL блога, с заменой всех пробельных символов на "_"
			* Проверка только в том случаи если создаём новый блог, т.к при редактировании URL нельзя менять
			*/		
			$teamUrl=preg_replace("/\s+/",'_',getRequest('team_url'));
			$_REQUEST['team_url']=$teamUrl;
			if (!func_check(getRequest('team_url'),'login',2,50)) {
				$this->Message_AddError($this->Lang_Get('team_create_url_error'),$this->Lang_Get('error'));
				$bOk=false;
			}
		}

		/**
		 * Проверяем есть ли уже блог с таким URL
		 */
		if ($oTeamExists=$this->PluginTeams_Teams_GetTeamByUrl(getRequest('team_url'))) {
			if (!$oTeam or $oTeam->getId()!=$oTeamExists->getId()) {
				$this->Message_AddError($this->Lang_Get('team_create_url_error_unique'),$this->Lang_Get('error'));
				$bOk=false;
			}
		}
		/**
		 * Проверяем есть ли описание блога
		 */
		if (!func_check(getRequest('team_description'),'text',10,3000)) {
			$this->Message_AddError($this->Lang_Get('team_create_description_error'),$this->Lang_Get('error'));
			$bOk=false;
		}
		/**
		 * Выполнение хуков
		 */
		$this->Hook_Run('check_team_fields', array('bOk'=>&$bOk));
			
		return $bOk;
	}
	
	protected function EventShowTeam() {
		$sTeamUrl=$this->sCurrentEvent;		
		
		/**
		 * Проверяем есть ли блог с таким УРЛ
		 */		
		if (!($oTeam=$this->PluginTeams_Teams_GetTeamByUrl($sTeamUrl))) {
			return parent::EventNotFound();
		}
					
		/**
		 * Меню
		 */
		$this->sMenuSubTeamUrl=$oTeam->getUrlFull();		
		
		/**
		 * Выставляем SEO данные
		 */
		$sTextSeo=preg_replace("/<.*>/Ui",' ',$oTeam->getDescription());
		$this->Viewer_SetHtmlDescription(func_text_words($sTextSeo,20));	
		/**
		 * Получаем список юзеров блога
		 */
		$aTeamUsers=$this->PluginTeams_Teams_GetTeamUsersByTeamId($oTeam->getId(),PluginTeams_Teams::TEAM_USER_ROLE_USER);
		$aTeamModerators=$this->PluginTeams_Teams_GetTeamUsersByTeamId($oTeam->getId(),PluginTeams_Teams::TEAM_USER_ROLE_MODERATOR);
		$aTeamAdministrators=$this->PluginTeams_Teams_GetTeamUsersByTeamId($oTeam->getId(),PluginTeams_Teams::TEAM_USER_ROLE_ADMINISTRATOR);
		
		/**
		 * Для админов проекта получаем список блогов и передаем их во вьювер
		 */
		if($this->oUserCurrent and $this->oUserCurrent->isAdministrator()) {
			$aTeams = $this->PluginTeams_Teams_GetTeams();
			unset($aTeams[$oTeam->getId()]);
			
			$this->Viewer_Assign('aTeams',$aTeams);
		}
		
		/**
		 * Загружаем переменные в шаблон
		 */				
		$this->Viewer_Assign('aTeamUsers',$aTeamUsers);		
		$this->Viewer_Assign('aTeamModerators',$aTeamModerators);
		$this->Viewer_Assign('aTeamAdministrators',$aTeamAdministrators);
		$this->Viewer_Assign('iCountTeamUsers',count($aTeamUsers));
		$this->Viewer_Assign('iCountTeamModerators',count($aTeamModerators));
		$this->Viewer_Assign('iCountTeamAdministrators',count($aTeamAdministrators)+1);		
		$this->Viewer_Assign('oTeam',$oTeam);
		$this->Viewer_AddHtmlTitle($oTeam->getTitle());
		/**
		 * Устанавливаем шаблон вывода
		 */
		$this->SetTemplateAction('team');
	}
	
	/**
	 * Обработка ajax запроса на отправку 
	 * пользователям приглашения вступить в закрытый блог
	 */
	protected function AjaxAddTeamInvite() {
		$this->Viewer_SetResponseAjax();
		$sUsers=getRequest('users',null,'post');
		$sTeamId=getRequest('idTeam',null,'post');
		
		/**
		 * Если пользователь не авторизирован, возвращаем ошибку
		 */
		if (!$this->User_IsAuthorization()) {	
			$this->Message_AddErrorSingle($this->Lang_Get('need_authorization'),$this->Lang_Get('error'));
			return;
		}
		$this->oUserCurrent=$this->User_GetUserCurrent();
		/**
		 * Проверяем существование блога
		 */
		if(!$oTeam=$this->PluginTeams_Teams_GetTeamById($sTeamId)) {
			$this->Message_AddErrorSingle($this->Lang_Get('system_error'),$this->Lang_Get('error'));
			return;			
		}
		/**
		 * Проверяем, имеет ли право текущий пользователь добавлять invite в blog
		 */
		$oTeamUser=$this->PluginTeams_Teams_GetTeamUserByTeamIdAndUserId($oTeam->getId(),$this->oUserCurrent->getId());		
		$bIsAdministratorTeam=$oTeamUser ? $oTeamUser->getIsAdministrator() : false;
		if ($oTeam->getOwnerId()!=$this->oUserCurrent->getId()  and !$this->oUserCurrent->isAdministrator() and !$bIsAdministratorTeam) {
			$this->Message_AddErrorSingle($this->Lang_Get('system_error'),$this->Lang_Get('error'));
			return;	
		}		
		
		/**
		 * Получаем список пользователей блога (любого статуса)		 
		 */
		$aTeamUsers = $this->PluginTeams_Teams_GetTeamUsersByTeamId(
			$oTeam->getId(),
			array(
				PluginTeams_Teams::TEAM_USER_ROLE_BAN,
				PluginTeams_Teams::TEAM_USER_ROLE_REJECT,
				PluginTeams_Teams::TEAM_USER_ROLE_INVITE,
				PluginTeams_Teams::TEAM_USER_ROLE_USER,
				PluginTeams_Teams::TEAM_USER_ROLE_MODERATOR,
				PluginTeams_Teams::TEAM_USER_ROLE_ADMINISTRATOR
			)
		);
		$aUsers=explode(',',$sUsers);

		$aResult=array();
		/**
		 * Обрабатываем добавление по каждому из переданных логинов
		 */
		foreach ($aUsers as $sUser) {
			$sUser=trim($sUser);
			if ($sUser=='') {
				continue;
			}
			/**
			 * Если пользователь пытается добавить инвайт 
			 * самому себе, возвращаем ошибку
			 */
			if(strtolower($sUser)==strtolower($this->oUserCurrent->getLogin())) {
				$aResult[]=array(
					'bStateError'=>true,
					'sMsgTitle'=>$this->Lang_Get('error'),
					'sMsg'=>$this->Lang_Get('team_user_invite_add_self')
				);										
				continue;
			}
			
			/**
			 * Если пользователь не найден или неактивен,
			 * возвращаем ошибку
			 */
			if (!$oUser=$this->User_GetUserByLogin($sUser) or $oUser->getActivate()!=1) {
				$aResult[]=array(
					'bStateError'=>true,
					'sMsgTitle'=>$this->Lang_Get('error'),
					'sMsg'=>$this->Lang_Get('user_not_found',array('login'=>$sUser)),
					'sUserLogin'=>$sUser
				);
				continue;
			}

			if(!isset($aTeamUsers[$oUser->getId()])) {
				/**
				 * Создаем нового блог-пользователя со статусом INVITED
				 */
				$oTeamUserNew=Engine::GetEntity('PluginTeams_Teams_TeamUser');
				$oTeamUserNew->setTeamId($oTeam->getId());
				$oTeamUserNew->setUserId($oUser->getId());
				$oTeamUserNew->setUserRole(PluginTeams_Teams::TEAM_USER_ROLE_INVITE);
				
				if($this->PluginTeams_Teams_AddRelationTeamUser($oTeamUserNew)) {
					$aResult[]=array(
						'bStateError'=>false,
						'sMsgTitle'=>$this->Lang_Get('attention'),
						'sMsg'=>$this->Lang_Get('team_user_invite_add_ok',array('login'=>$sUser)),
						'sUserLogin'=>$sUser,
						'sUserWebPath'=>$oUser->getUserWebPath()
					);
					$this->SendTeamInvite($oTeam,$oUser);
				} else {
					$aResult[]=array(
						'bStateError'=>true,
						'sMsgTitle'=>$this->Lang_Get('error'),
						'sMsg'=>$this->Lang_Get('system_error'),
						'sUserLogin'=>$sUser
					);					
				}
			} else {
				/**
				 * Попытка добавить приглашение уже существующему пользователю,
				 * возвращаем ошибку (сначала определяя ее точный текст)
				 */
				switch (true) {
					case ($aTeamUsers[$oUser->getId()]->getUserRole()==PluginTeams_Teams::TEAM_USER_ROLE_INVITE):
						$sErrorMessage=$this->Lang_Get('team_user_already_invited',array('login'=>$sUser));
						break;
					case ($aTeamUsers[$oUser->getId()]->getUserRole()>PluginTeams_Teams::TEAM_USER_ROLE_GUEST):
						$sErrorMessage=$this->Lang_Get('team_user_already_exists',array('login'=>$sUser));						
						break;
					case ($aTeamUsers[$oUser->getId()]->getUserRole()==PluginTeams_Teams::TEAM_USER_ROLE_REJECT):
						$sErrorMessage=$this->Lang_Get('team_user_already_reject',array('login'=>$sUser));						
						break;
					default:
						$sErrorMessage=$this->Lang_Get('system_error');
				}
				$aResult[]=array(
					'bStateError'=>true,
					'sMsgTitle'=>$this->Lang_Get('error'),
					'sMsg'=>$sErrorMessage,
					'sUserLogin'=>$sUser
				);
				continue;
			}		
		}
		
		/**
		 * Передаем во вьевер массив с результатами обработки по каждому пользователю
		 */
		$this->Viewer_AssignAjax('aUsers',$aResult);
	}

	/**
	 * Выполняет отправку приглашения в блог 
	 * (по внутренней почте и на email)
	 *
	 * @param BlogEntity_Blog $oBlog
	 * @param UserEntity_User $oUser
	 */
	protected function SendTeamInvite($oTeam,$oUser) {
		
		$sTitle=$this->Lang_Get(
			'team_user_invite_title',
			array(
				'team_title'=>$oTeam->getTitle()
			)
		);

		require_once Config::Get('path.root.engine').'/lib/external/XXTEA/encrypt.php';
		$sCode=$oTeam->getId().'_'.$oUser->getId();
		$sCode=urlencode(xxtea_encrypt($sCode, Config::Get('module.blog.encrypt')));

		$aPath=array(
			'accept'=>Router::GetPath('teams').'invite/accept/?code='.$sCode,
			'reject'=>Router::GetPath('teams').'invite/reject/?code='.$sCode
		);

		$sText=$this->Lang_Get(
			'team_user_invite_text',
			array(
				'login'=>$this->oUserCurrent->getLogin(),
				'accept_path'=>$aPath['accept'],
				'reject_path'=>$aPath['reject'],
				'team_title'=>$oTeam->getTitle()
			)
		);
		$oTalk=$this->Talk_SendTalk($sTitle,$sText,$this->oUserCurrent,array($oUser),false,false);
		/**
		 * Отправляем пользователю заявку
		 */
		$this->Notify_SendTeamUserInvite(
			$oUser,$this->oUserCurrent,$oTeam,
			Router::GetPath('talk').'read/'.$oTalk->getId().'/'
		);
		/**
		 * Удаляем отправляющего юзера из переписки
		 */	
		$this->Talk_DeleteTalkUserByArray($oTalk->getId(),$this->oUserCurrent->getId());
	}
	
	/**
	 * Обработка отправленого пользователю приглашения вступить в блог
	 */
	protected function EventInviteTeam() {	
		require_once Config::Get('path.root.engine').'/lib/external/XXTEA/encrypt.php';
		$sCode=xxtea_decrypt(urldecode(getRequest('code')), Config::Get('module.blog.encrypt'));

		list($sTeamId,$sUserId)=explode('_',$sCode,2);
		
		$sAction=$this->GetParam(0);
		
		/**
		 * Получаем текущего пользователя
		 */
		if(!$this->User_IsAuthorization()) {
			return $this->EventNotFound();
		}
		$this->oUserCurrent = $this->User_GetUserCurrent();
		/**
		 * Если приглашенный пользователь не является авторизированным
		 */
		if($this->oUserCurrent->getId()!=$sUserId) {
			return $this->EventNotFound();
		}
		/**
		 * Получаем указанный блог
		 */
		if (!$oTeam=$this->PluginTeams_Teams_GetTeamById($sTeamId))  {
			return $this->EventNotFound();
		}
				
		/**
		 * Получаем связь "блог-пользователь" и проверяем,
		 * чтобы ее тип был INVITE или REJECT
		 */
		if(!$oTeamUser=$this->PluginTeams_Teams_GetTeamUserByTeamIdAndUserId($oTeam->getId(),$this->oUserCurrent->getId())) {
			return $this->EventNotFound();
		}
		if($oTeamUser->getUserRole()>PluginTeams_Teams::TEAM_USER_ROLE_GUEST) {
			$sMessage=$this->Lang_Get('team_user_invite_already_done');
			$this->Message_AddError($sMessage,$this->Lang_Get('error'),true);
			Router::Location(Router::GetPath('talk'));
			return ;						
		}
		if(!in_array($oTeamUser->getUserRole(),array(PluginTeams_Teams::TEAM_USER_ROLE_INVITE,PluginTeams_Teams::TEAM_USER_ROLE_REJECT))) {
			$this->Message_AddError($this->Lang_Get('system_error'),$this->Lang_Get('error'),true);
			Router::Location(Router::GetPath('talk'));
			return ;
		}
		
		/**
		 * Обновляем роль пользователя до читателя
		 */
		$oTeamUser->setUserRole(($sAction=='accept')?PluginTeams_Teams::TEAM_USER_ROLE_USER:PluginTeams_Teams::TEAM_USER_ROLE_REJECT);
		if(!$this->PluginTeams_Teams_UpdateRelationTeamUser($oTeamUser)) {
			$this->Message_AddError($this->Lang_Get('system_error'),$this->Lang_Get('error'),true);
			Router::Location(Router::GetPath('talk'));
			return ;						
		} 
		$sMessage = ($sAction=='accept')
			? $this->Lang_Get('team_user_invite_accept')
			: $this->Lang_Get('team_user_invite_reject');
		$this->Message_AddNotice($sMessage,$this->Lang_Get('attention'),true);
		
		Router::Location(Router::GetPath('talk'));		
	}

	/**
	 * Удаление блога
	 *
	 * @return bool
	 */
	protected function EventDeleteTeam() {
		$this->Security_ValidateSendForm();
		/**
		 * Проверяем передан ли в УРЛе номер блога
		 */
		$sTeamId=$this->GetParam(0);
		if (!$oTeam=$this->PluginTeams_Teams_GetTeamById($sTeamId)) {
			return parent::EventNotFound();
		}
		/**
		 * Проверям авторизован ли пользователь
		 */
		if (!$this->User_IsAuthorization()) {
			$this->Message_AddErrorSingle($this->Lang_Get('not_access'),$this->Lang_Get('error'));
			return Router::Action('error');
		}
		
		/**
		 * Проверяем является ли юзер администратором
		 */
		if (!$this->oUserCurrent->isAdministrator()) {
			$this->Message_AddErrorSingle($this->Lang_Get('not_access'));
			return Router::Action('error');
		}
		/**
		 * Удаляяем блог и перенаправляем пользователя к списку блогов
		 */
		$this->Hook_Run('team_delete_before', array('sTeamId'=>$sTeamId));		
		if($this->PluginTeams_Teams_DeleteTeam($sTeamId)) {
			$this->Hook_Run('team_delete_after', array('sTeamId'=>$sTeamId));			
			$this->Message_AddNoticeSingle($this->Lang_Get('team_admin_delete_success'),$this->Lang_Get('attention'),true);
			Router::Location(Router::GetPath('teams'));
		} else {
			Router::Location($oTeam()->getUrlFull());
		}
	}
	
	/**
	 * Завершение работы Action`a
	 *
	 */
	public function EventShutdown() {

	}
}
?>