<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

set_include_path(get_include_path().PATH_SEPARATOR.dirname(__FILE__));
require_once('mapper/Teams.mapper.class.php');

/**
 * Модуль статических страниц
 *
 */
class PluginTeams_Teams extends Module {		
	/**
	 * Меппер для сохранения логов в базу данных и формирования выборок по данным из базы
	 *
	 * @var Mapper_Profiler
	 */
	const TEAM_USER_ROLE_GUEST         = 0;
	const TEAM_USER_ROLE_USER          = 1;
	const TEAM_USER_ROLE_MODERATOR     = 2;
	const TEAM_USER_ROLE_ADMINISTRATOR = 4;
	/**
	 * Пользователь, приглашенный админом блога в блог
	 */
	const TEAM_USER_ROLE_INVITE        = -1;
	/**
	 * Пользователь, отклонивший приглашение админа
	 */
	const TEAM_USER_ROLE_REJECT        = -2;
	/**
	 * Забаненный в блоге пользователь
	 */
	const TEAM_USER_ROLE_BAN           = -4;
	
	protected $oMapper;	
	protected $oUserCurrent=null;
	
	/**
	 * Инициализация модуля
	 */
	public function Init() {
		
		$this->oMapper=new PluginTeams_Mapper_Teams($this->Database_GetConnect());
		$this->oMapper->SetUserCurrent($this->User_GetUserCurrent());
		$this->oUserCurrent=$this->User_GetUserCurrent();		
	}
	
/**
	 * Получает дополнительные данные(объекты) для блогов по их ID
	 *
	 */
	public function GetTeamsAdditionalData($aTeamId,$aAllowData=array('owner'=>array(),'relation_user')) {
		func_array_simpleflip($aAllowData);
		if (!is_array($aTeamId)) {
			$aTeamId=array($aTeamId);
		}
		/**
		 * Получаем блоги
		 */
		$oTeam = Engine::GetEntity('PluginTeams_Teams_Teams');
		$aTeams=$this->GetTeamsByArrayId($aTeamId);
		/**
		 * Формируем ID дополнительных данных, которые нужно получить
		 */
		$aUserId=array();		
		foreach ($aTeams as $oTeam) {
			if (isset($aAllowData['owner'])) {
				$aUserId[]=$oTeam->getOwnerId();
			}						
		}
		/**
		 * Получаем дополнительные данные
		 */
		$aTeamUsers = Engine::GetEntity('PluginTeams_Teams_TeamUser');
		$aTeamUsers=array();
		$aUsers=isset($aAllowData['owner']) && is_array($aAllowData['owner']) ? $this->User_GetUsersAdditionalData($aUserId,$aAllowData['owner']) : $this->User_GetUsersAdditionalData($aUserId);				
		if (isset($aAllowData['relation_user']) and $this->oUserCurrent) {
			$aTeamUsers=$this->GetTeamUsersByArrayTeam($aTeamId,$this->oUserCurrent->getId());	
		}

		/**
		 * Добавляем данные к результату - списку блогов
		 */
		foreach ($aTeams as $oTeam) {
			if (isset($aUsers[$oTeam->getOwnerId()])) {
				$oTeam->setOwner($aUsers[$oTeam->getOwnerId()]);
			} else {
				$oTeam->setOwner(null); // или $oBlog->setOwner(new UserEntity_User());
			}
			if (isset($aTeamUsers[$oTeam->getId()])) {
				$oTeam->setUserIsJoin(true);
				if ($aTeamUsers[$oTeam->getId()]->getIsAdministrator()) {
					$oTeam->setUserIsAdministrator(true);
				}
				if ($aTeamUsers[$oTeam->getId()]->getIsModerator()) {
					$oTeam->setUserIsModerator(true);
				}
			} else {
				$oTeam->setUserIsJoin(false);
				$oTeam->setUserIsAdministrator(false);
				$oTeam->setUserIsModerator(false);
			}				
					
		}
		
		return $aTeams;
	}
	/**
	 * Список блогов по ID
	 *
	 * @param array $aUserId
	 */
	public function GetTeamsByArrayId($aTeamId) {
		if (!$aTeamId) {
			return array();
		}
		if (Config::Get('sys.cache.solid')) {
			return $this->GetTeamsByArrayIdSolid($aTeamId);
		}
		if (!is_array($aTeamId)) {
			$aTeamId=array($aTeamId);
		}
		$aTeamId=array_unique($aTeamId);
		$aTeams=array();
		$aTeamIdNotNeedQuery=array();
		/**
		 * Делаем мульти-запрос к кешу
		 */
		$aCacheKeys=func_build_cache_keys($aTeamId,'team_');
		if (false !== ($data = $this->Cache_Get($aCacheKeys))) {			
			/**
			 * проверяем что досталось из кеша
			 */
			foreach ($aCacheKeys as $sValue => $sKey ) {
				if (array_key_exists($sKey,$data)) {	
					if ($data[$sKey]) {
						$aTeams[$data[$sKey]->getId()]=$data[$sKey];
					} else {
						$aTeamIdNotNeedQuery[]=$sValue;
					}
				} 
			}
		}
		/**
		 * Смотрим каких блогов не было в кеше и делаем запрос в БД
		 */		
		$aTeamIdNeedQuery=array_diff($aTeamId,array_keys($aTeams));		
		$aTeamIdNeedQuery=array_diff($aTeamIdNeedQuery,$aTeamIdNotNeedQuery);		
		$aTeamIdNeedStore=$aTeamIdNeedQuery;
		if ($data = $this->oMapper->GetTeamsByArrayId($aTeamIdNeedQuery)) {
			foreach ($data as $oTeam) {
				/**
				 * Добавляем к результату и сохраняем в кеш
				 */
				$aTeams[$oTeam->getId()]=$oTeam;
				$this->Cache_Set($oTeam, "team_{$oTeam->getId()}", array(), 60*60*24*4);
				$aTeamIdNeedStore=array_diff($aTeamIdNeedStore,array($oTeam->getId()));
			}
		}
		/**
		 * Сохраняем в кеш запросы не вернувшие результата
		 */
		foreach ($aTeamIdNeedStore as $sId) {
			$this->Cache_Set(null, "team_{$sId}", array(), 60*60*24*4);
		}		
		/**
		 * Сортируем результат согласно входящему массиву
		 */
		$aTeams=func_array_sort_by_keys($aTeams,$aTeamId);
		return $aTeams;		
	}
	/**
	 * Список блогов по ID, но используя единый кеш
	 *
	 * @param unknown_type $aBlogId
	 * @return unknown
	 */
	public function GetTeamsByArrayIdSolid($aTeamId) {
		if (!is_array($aTeamId)) {
			$aTeamId=array($aTeamId);
		}
		$aTeamId=array_unique($aTeamId);	
		$aTeams=array();	
		$s=join(',',$aTeamId);
		if (false === ($data = $this->Cache_Get("team_id_{$s}"))) {			
			$data = $this->oMapper->GetTeamsByArrayId($aTeamId);
			foreach ($data as $oTeam) {
				$aTeams[$oTeam->getId()]=$oTeam;
			}
			$this->Cache_Set($aTeams, "team_id_{$s}", array("team_update"), 60*60*24*1);
			return $aTeams;
		}		
		return $data;
	}
	/**
	 * Получить блог по айдишнику(номеру)
	 *
	 * @param unknown_type $sBlogId
	 * @return unknown
	 */
	public function GetTeamById($sTeamId) {
		$aTeams=$this->GetTeamsAdditionalData($sTeamId);
		if (isset($aTeams[$sTeamId])) {
			return $aTeams[$sTeamId];
		}
		return null;		
	}
	/**
	 * Получить блог по УРЛу
	 *
	 * @param unknown_type $sBlogUrl
	 * @return unknown
	 */
	public function GetTeamByUrl($sTeamUrl) {		
		if (false === ($id = $this->Cache_Get("team_url_{$sTeamUrl}"))) {						
			if ($id = $this->oMapper->GetTeamByUrl($sTeamUrl)) {				
				$this->Cache_Set($id, "team_url_{$sTeamUrl}", array("team_update_{$id}"), 60*60*24*2);				
			} else {
				$this->Cache_Set(null, "team_url_{$sTeamUrl}", array('team_update','team_new'), 60*60);
			}
		}		
		return $this->GetTeamById($id);		
	}
	/**
	 * Получить блог по названию
	 *
	 * @param unknown_type $sTitle
	 * @return unknown
	 */
	public function GetTeamByTitle($sTitle) {		
		if (false === ($id = $this->Cache_Get("team_title_{$sTitle}"))) {						
			if ($id = $this->oMapper->GetTeamByTitle($sTitle)) {				
				$this->Cache_Set($id, "team_title_{$sTitle}", array("team_update_{$id}",'team_new'), 60*60*24*2);				
			} else {
				$this->Cache_Set(null, "team_title_{$sTitle}", array('team_update','team_new'), 60*60);
			}
		}
		return $this->GetTeamById($id);		
	}
	
	/**
	 * Добавляет блог
	 *
	 * @param BlogEntity_Blog $oBlog
	 * @return unknown
	 */
	public function AddTeam(PluginTeams_TeamsEntity_Teams $oTeam) {		
		if ($sId=$this->oMapper->AddTeam($oTeam)) {
			$oTeam->setId($sId);
			//чистим зависимые кеши
			$this->Cache_Clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,array('team_new',"team_new_user_{$oTeam->getOwnerId()}"));						
			return $oTeam;
		}
		return false;
	}
	/**
	 * Обновляет блог
	 *
	 * @param BlogEntity_Blog $oBlog
	 * @return unknown
	 */
	public function UpdateTeam(PluginTeams_TeamsEntity_Teams $oTeam) {
		$oTeam->setDateEdit(date("Y-m-d H:i:s"));
		$res=$this->oMapper->UpdateTeam($oTeam);		
		if ($res) {			
			//чистим зависимые кеши
			$this->Cache_Clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,array('team_update',"team_update_{$oTeam->getId()}"));
			$this->Cache_Delete("team_{$oTeam->getId()}");
			return true;
		}
		return false;
	}	
	/**
	 * Добавляет отношение юзера к блогу, по сути присоединяет к блогу
	 *
	 * @param BlogEntity_BlogUser $oBlogUser
	 * @return unknown
	 */
	public function AddRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {
		if ($this->oMapper->AddRelationTeamUser($oTeamUser)) {		
			$this->Cache_Clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,array("team_relation_change_{$oTeamUser->getUserId()}","team_relation_change_team_{$oTeamUser->getTeamId()}"));	
			$this->Cache_Delete("team_relation_user_{$oTeamUser->getTeamId()}_{$oTeamUser->getUserId()}");	
			return true;
		}
		return false;
	}
	/**
	 * Удалет отношение юзера к блогу, по сути отключает от блога
	 *
	 * @param BlogEntity_BlogUser $oBlogUser
	 * @return unknown
	 */
	public function DeleteRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {
		if ($this->oMapper->DeleteRelationTeamUser($oTeamUser)) {
			$this->Cache_Clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,array("team_relation_change_{$oTeamUser->getUserId()}","team_relation_change_team_{$oTeamUser->getTeamId()}"));		
			$this->Cache_Delete("team_relation_user_{$oTeamUser->getTeamId()}_{$oTeamUser->getUserId()}");
			return true;
		}
		return false;
	}
	/**
	 * Получает список блогов по хозяину
	 *
	 * @param unknown_type $sUserId
	 * @return unknown
	 */
	public function GetTeamsByOwnerId($sUserId,$bReturnIdOnly=false) {
		$data=$this->oMapper->GetTeamsByOwnerId($sUserId);
		/**
		 * Возвращаем только иденитификаторы
		 */
		if($bReturnIdOnly) return $data;
		
		$data=$this->GetTeamsAdditionalData($data);
		return $data;
	}
	/**
	 * Получает список всех НЕ персональных блогов
	 *
	 * @return unknown
	 */
	public function GetTeams() {
		$data=$this->oMapper->GetTeams();
		/**
		 * Возвращаем только иденитификаторы
		 */
					
		$data=$this->GetTeamsAdditionalData($data);
		return $data;
	}
		
	/**
	 * Получает список пользователей блога.
	 * Если роль не указана, то считаем что 
	 * поиск производиться по положительным значениям
	 * (статусом выше GUEST).
	 *
	 * @param  string           $sBlogId
	 * @param  (null|int|array) $iRole
	 * @return array
	 */
	public function GetTeamUsersByTeamId($sTeamId,$iRole=null) {
		$aFilter=array(
			'team_id'=> $sTeamId,			
		);
		if($iRole!==null) {
			$aFilter['user_role']=$iRole;	
		}
		$s=serialize($aFilter);
		if (false === ($data = $this->Cache_Get("team_relation_user_by_filter_$s"))) {				
			$data = $this->oMapper->GetTeamUsers($aFilter);
			$this->Cache_Set($data, "team_relation_user_by_filter_$s", array("team_relation_change_team_{$sTeamId}"), 60*60*24*3);
		}
		/**
		 * Достаем дополнительные данные, для этого формируем список юзеров и делаем мульти-запрос
		 */
		 $oTeamUser = Engine::GetEntity('PluginTeams_Teams_TeamUser');
		if ($data) {
			$aUserId=array();
			foreach ($data as $oTeamUser) {
				$aUserId[]=$oTeamUser->getUserId();
			}
			$aUsers=$this->User_GetUsersAdditionalData($aUserId);
			$aTeams=$this->PluginTeams_Teams_GetTeamsAdditionalData($sTeamId);
			
			$aResults=array();
			foreach ($data as $oTeamUser) {
				if (isset($aUsers[$oTeamUser->getUserId()])) {
					$oTeamUser->setUser($aUsers[$oTeamUser->getUserId()]);
				} else {
					$oTeamUser->setUser(null);
				}
				if (isset($aTeams[$oTeamUser->getTeamId()])) {
					$oTeamUser->setTeam($aTeams[$oTeamUser->getTeamId()]);
				} else {
					$oTeamUser->setTeam(null);
				}
				$aResults[$oTeamUser->getUserId()]=$oTeamUser;
			}
			$data=$aResults;
		}
		return $data;		
	}
	/**
	 * Получает отношения юзера к блогам(состоит в блоге или нет)
	 *
	 * @param unknown_type $sUserId
	 * @return unknown
	 */
	public function GetTeamUsersByUserId($sUserId,$iRole=null,$bReturnIdOnly=false) {
		$aFilter=array(
			'user_id'=> $sUserId
		);
		if($iRole!==null) {
			$aFilter['user_role']=$iRole;
		}
		$oTeamUser = Engine::GetEntity('PluginTeams_Teams_TeamUser');
		$s=serialize($aFilter);
		if (false === ($data = $this->Cache_Get("team_relation_user_by_filter_$s"))) {				
			$data = $this->oMapper->GetTeamUsers($aFilter);
			$this->Cache_Set($data, "team_relation_user_by_filter_$s", array("team_relation_change_{$sUserId}"), 60*60*24*3);
		}
		/**
		 * Достаем дополнительные данные, для этого формируем список блогов и делаем мульти-запрос
		 */
		$aTeamId=array();		
		if ($data) {
		
			foreach ($data as $oTeamUser) {
				$aTeamId[]=$oTeamUser->getTeamId();
			}
			
			/**
			 * Если указано возвращать полные объекты
			 */
			if(!$bReturnIdOnly) {
				$aUsers=$this->User_GetUsersAdditionalData($sUserId);
				$aTeams=$this->PluginTeams_Teams_GetTeamsAdditionalData($aTeamId);
				foreach ($data as $oTeamUser) {
					if (isset($aUsers[$oTeamUser->getUserId()])) {
						$oTeamUser->setUser($aUsers[$oTeamUser->getUserId()]);
					} else {
						$oTeamUser->setUser(null);
					}
					if (isset($aTeams[$oTeamUser->getTeamId()])) {
						$oTeamUser->setTeam($aTeams[$oTeamUser->getTeamId()]);
					} else {
						$oTeamUser->setTeam(null);
					}
				}				
			}
		}
		return ($bReturnIdOnly) ? $aTeamId : $data;
	}
	/**
	 * Состоит ли юзер в конкретном блоге
	 *
	 * @param unknown_type $sBlogId
	 * @param unknown_type $sUserId
	 * @return unknown
	 */
	public function GetTeamUserByTeamIdAndUserId($sTeamId,$sUserId) {		
		if ($aTeamUser=$this->GetTeamUsersByArrayTeam($sTeamId,$sUserId)) {
			if (isset($aTeamUser[$sTeamId])) {
				return $aTeamUser[$sTeamId];
			}
		}
		return null;
	}
	/**
	 * Получить список отношений блог-юзер по списку айдишников
	 *
	 * @param unknown_type $aTopicId
	 */
	public function GetTeamUsersByArrayTeam($aTeamId,$sUserId) {
		if (!$aTeamId) {
			return array();
		}
		if (Config::Get('sys.cache.solid')) {
			return $this->GetTeamUsersByArrayTeamSolid($aTeamId,$sUserId);
		}
		if (!is_array($aTeamId)) {
			$aTeamId=array($aTeamId);
		}
		$aTeamId=array_unique($aTeamId);
		$aTeamUsers=array();
		$aTeamIdNotNeedQuery=array();
		/**
		 * Делаем мульти-запрос к кешу
		 */
		$aCacheKeys=func_build_cache_keys($aTeamId,'team_relation_user_','_'.$sUserId);
		if (false !== ($data = $this->Cache_Get($aCacheKeys))) {			
			/**
			 * проверяем что досталось из кеша
			 */
			foreach ($aCacheKeys as $sValue => $sKey ) {
				if (array_key_exists($sKey,$data)) {	
					if ($data[$sKey]) {
						$aTeamUsers[$data[$sKey]->getTeamId()]=$data[$sKey];
					} else {
						$aTeamIdNotNeedQuery[]=$sValue;
					}
				} 
			}
		}
		/**
		 * Смотрим каких блогов не было в кеше и делаем запрос в БД
		 */		
		$aTeamIdNeedQuery=array_diff($aTeamId,array_keys($aTeamUsers));		
		$aTeamIdNeedQuery=array_diff($aTeamIdNeedQuery,$aTeamIdNotNeedQuery);		
		$aTeamIdNeedStore=$aTeamIdNeedQuery;
		if ($data = $this->oMapper->GetTeamUsersByArrayTeam($aTeamIdNeedQuery,$sUserId)) {
			foreach ($data as $oTeamUser) {
				/**
				 * Добавляем к результату и сохраняем в кеш
				 */
				$aTeamUsers[$oTeamUser->getTeamId()]=$oTeamUser;
				$this->Cache_Set($oTeamUser, "team_relation_user_{$oTeamUser->getTeamId()}_{$oTeamUser->getUserId()}", array(), 60*60*24*4);
				$aTeamIdNeedStore=array_diff($aTeamIdNeedStore,array($oTeamUser->getTeamId()));
			}
		}
		/**
		 * Сохраняем в кеш запросы не вернувшие результата
		 */
		foreach ($aTeamIdNeedStore as $sId) {
			$this->Cache_Set(null, "team_relation_user_{$sId}_{$sUserId}", array(), 60*60*24*4);
		}		
		/**
		 * Сортируем результат согласно входящему массиву
		 */
		$aTeamUsers=func_array_sort_by_keys($aTeamUsers,$aTeamId);
		return $aTeamUsers;		
	}	
	public function GetTeamUsersByArrayTeamSolid($aTeamId,$sUserId) {
		if (!is_array($aTeamId)) {
			$aTeamId=array($aTeamId);
		}
		$aTeamId=array_unique($aTeamId);	
		$aTeamUsers=array();	
		$s=join(',',$aTeamId);
		if (false === ($data = $this->Cache_Get("team_relation_user_{$sUserId}_id_{$s}"))) {			
			$data = $this->oMapper->GetTeamUsersByArrayTeam($aTeamId,$sUserId);
			foreach ($data as $oTeamUser) {
				$aTeamUsers[$oTeamUser->getTeamId()]=$oTeamUser;
			}
			$this->Cache_Set($aTeamUsers, "team_relation_user_{$sUserId}_id_{$s}", array("team_relation_change_{$sUserId}"), 60*60*24*1);
			return $aTeamUsers;
		}		
		return $data;
	}
	public function UpdateRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {
		$this->Cache_Clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG,array("team_relation_change_{$oTeamUser->getUserId()}","team_relation_change_team_{$oTeamUser->getTeamId()}"));		
		$this->Cache_Delete("team_relation_user_{$oTeamUser->getTeamId()}_{$oTeamUser->getUserId()}");
		return $this->oMapper->UpdateRelationTeamUser($oTeamUser);
	}
	
	/**
	 * Удаляет блог
	 *
	 * @param  int $iBlogId
	 * @return bool
	 */
	public function DeleteTeam($iTeamId) {
		if($iTeamId instanceof PluginTeams_TeamsEntity_Teams){
			$iTeamId = $iTeamId->getId();
		}
				/**
		 * Если блог не удален, возвращаем false
		 */
		if(!$this->oMapper->DeleteTeam($iTeamId)) { return false; }
		/**
		 * Чистим кеш
		 */
		$this->Cache_Clean(
			Zend_Cache::CLEANING_MODE_MATCHING_TAG,
			array(
				"team_update", "team_relation_change_team_{$iTeamId}"				
			)
		);
		$this->Cache_Delete("team_{$iTeamId}");
		
		/**
		 * Удаляем связи пользователей блога.
		 */
		if(Config::Get('db.tables.engine')!="InnoDB"){ 
			$this->oMapper->DeleteTeamUsersByTeamId($iTeamId);
		}
		/**
		 * Удаляем комментарии к записям из блога и метки прямого эфира
		 */
		
		return true;
	}
	/**
	 * Upload blog avatar on server
	 * Make resized images
	 *
	 * @param  array           $aFile
	 * @param  BlogEntity_Blog $oUser
	 * @return (string|bool)
	 */
	public function UploadTeamAvatar($aFile,$oTeam) {
		if(!is_array($aFile) || !isset($aFile['tmp_name'])) {
			return false;
		}
		
		$sFileTmp=$aFile['tmp_name'];
		$sPath=$this->Image_GetIdDir($oTeam->getOwnerId());
		$aParams=$this->Image_BuildParams('foto');

		$oImage=new LiveImage($sFileTmp);
		/**
		 * Если объект изображения не создан, 
		 * возвращаем ошибку
		 */
		if($sError=$oImage->get_last_error()) {
			// Вывод сообщения об ошибки, произошедшей при создании объекта изображения
			// $this->Message_AddError($sError,$this->Lang_Get('error'));
			return false;
		}		
		/**
		 * Срезаем квадрат
		 */
		
		if ($oImage && $sFileAvatar=$this->Image_Resize($sFileTmp,$sPath,"avatar_team_{$oTeam->getUrl()}",3000,3000,450,null,true,$aParams,$oImage)) {
			
			
			/**
			 * Если все нормально, возвращаем расширение загруженного аватара
			 */
			return $this->Image_GetWebPath($sFileAvatar);
		}
		/**
		 * В случае ошибки, возвращаем false
		 */
		return false;
	}
	/**
	 * Delete blog avatar from server
	 *
	 * @param BlogEntity_Blog $oUser
	 */
	public function DeleteTeamAvatar($oTeam) {
		/**
		 * Если аватар есть, удаляем его и его рейсайзы
		 */
		if($oTeam->getAvatar()) {		
			@unlink($this->Image_GetServerPath($oTeam->getAvatarPath()));		
		}
	}	
}
?>