<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

class PluginTeams_Mapper_Teams extends Mapper {	

	protected $oUserCurrent=null;
	
	public function SetUserCurrent($oUserCurrent)  {
		$this->oUserCurrent=$oUserCurrent;
	}
	
	public function AddTeam(PluginTeams_TeamsEntity_Teams $oTeam) {
		$sql = "INSERT INTO ".Config::Get('db.table.teams')." 
			(user_owner_id,
			team_title,
			team_description,
			team_date_add,
			team_url,
			team_avatar
			)
			VALUES(?d,  ?,	?,	?,	?,	?)
		";			
		if ($iId=$this->oDb->query($sql,$oTeam->getOwnerId(),$oTeam->getTitle(),$oTeam->getDescription(),$oTeam->getDateAdd(),$oTeam->getUrl(),$oTeam->getAvatar())) {
			return $iId;
		}		
		return false;
	}
	
	public function UpdateTeam(PluginTeams_TeamsEntity_Teams $oTeam) {		
		$sql = "UPDATE ".Config::Get('db.table.teams')." 
			SET 
				team_title= ?,
				team_description= ?,
				team_date_edit= ?,
				team_url= ?,
				team_avatar= ?
			WHERE
				team_id = ?d
		";			
		if ($this->oDb->query($sql,$oTeam->getTitle(),$oTeam->getDescription(),$oTeam->getDateEdit(),$oTeam->getUrl(),$oTeam->getAvatar(),$oTeam->getId())) {
			return true;
		}		
		return false;
	}
	
	
	public function GetTeamsByArrayId($aArrayId) {
		if (!is_array($aArrayId) or count($aArrayId)==0) {
			return array();
		}
				
		$sql = "SELECT 
					b.*							 
				FROM 
					".Config::Get('db.table.teams')." as b					
				WHERE 
					b.team_id IN(?a) 								
				ORDER BY FIELD(b.team_id,?a) ";
		$aTeams=array();
		if ($aRows=$this->oDb->select($sql,$aArrayId,$aArrayId)) {
			foreach ($aRows as $aTeam) {
				$aTeams[]=Engine::GetEntity('PluginTeams_Teams_Teams',$aTeam);
			}
		}
		return $aTeams;
	}	
	
	public function AddRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {
		$sql = "INSERT INTO ".Config::Get('db.table.team_user')." 
			(team_id,
			user_id,
			user_role
			)
			VALUES(?d,  ?d, ?d)
		";			
		if ($this->oDb->query($sql,$oTeamUser->getTeamId(),$oTeamUser->getUserId(),$oTeamUser->getUserRole())===0) {
			return true;
		}		
		return false;
	}
	
	public function DeleteRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {
		$sql = "DELETE FROM ".Config::Get('db.table.team_user')." 
			WHERE
				team_id = ?d
				AND
				user_id = ?d
		";			
		if ($this->oDb->query($sql,$oTeamUser->getTeamId(),$oTeamUser->getUserId())) {
			return true;
		}		
		return false;
	}
		
	public function UpdateRelationTeamUser(PluginTeams_TeamsEntity_TeamUser $oTeamUser) {		
		$sql = "UPDATE ".Config::Get('db.table.team_user')." 
			SET 
				user_role = ?d			
			WHERE
				team_id = ?d 
				AND
				user_id = ?d
		";			
		if ($this->oDb->query($sql,$oTeamUser->getUserRole(),$oTeamUser->getTeamId(),$oTeamUser->getUserId())) {
			return true;
		}		
		return false;
	}
	
	public function GetTeamUsers($aFilter) {
		$sWhere=' 1=1 ';
		if (isset($aFilter['team_id'])) {
			$sWhere.=" AND bu.team_id =  ".(int)$aFilter['team_id'];
		}
		if (isset($aFilter['user_id'])) {
			$sWhere.=" AND bu.user_id =  ".(int)$aFilter['user_id'];
		}
		if (isset($aFilter['user_role'])) {
			if(!is_array($aFilter['user_role'])) {
				$aFilter['user_role']=array($aFilter['user_role']);
			}
			$sWhere.=" AND bu.user_role IN ('".join("', '",$aFilter['user_role'])."')";		
		} else {
			$sWhere.=" AND bu.user_role>".PluginTeams_Teams::TEAM_USER_ROLE_GUEST;
		}
		
		$sql = "SELECT
					bu.*				
				FROM 
					".Config::Get('db.table.team_user')." as bu
				WHERE 
					".$sWhere." 					
				;
					";		
		$aTeamUsers=array();
		if ($aRows=$this->oDb->select($sql)) {
			foreach ($aRows as $aUser) {
				$aTeamUsers[]=Engine::GetEntity('PluginTeams_Teams_TeamUser',$aUser);
			}
		}
		return $aTeamUsers;
	}

	public function GetTeamUsersByArrayTeam($aArrayId,$sUserId) {	
		if (!is_array($aArrayId) or count($aArrayId)==0) {
			return array();
		}
			
		$sql = "SELECT 
					bu.*				
				FROM 
					".Config::Get('db.table.team_user')." as bu
				WHERE 
					bu.user_id = ?d
					AND
					bu.team_id IN(?a) ";		
		$aTeamUsers=array();
		if ($aRows=$this->oDb->select($sql,$sUserId,$aArrayId)) {
			foreach ($aRows as $aUser) {
				$aTeamUsers[]=Engine::GetEntity('PluginTeams_Teams_TeamUser',$aUser);
			}
		}
		return $aTeamUsers;
	}
	
		
	public function GetTeamByTitle($sTitle) {
		$sql = "SELECT team_id FROM ".Config::Get('db.table.teams')." WHERE team_title = ? ";
		if ($aRow=$this->oDb->selectRow($sql,$sTitle)) {
			return $aRow['team_id'];
		}
		return null;
	}
	
	
	
	public function GetTeamByUrl($sUrl) {		
		$sql = "SELECT 
				b.team_id 
			FROM 
				".Config::Get('db.table.teams')." as b
			WHERE 
				b.team_url = ? 		
				";
		if ($aRow=$this->oDb->selectRow($sql,$sUrl)) {
			return $aRow['team_id'];
		}
		return null;
	}
	
	public function GetTeamsByOwnerId($sUserId) {
		$sql = "SELECT 
			b.team_id			 
			FROM 
				".Config::Get('db.table.teams')." as b				
			WHERE 
				b.user_owner_id = ? 
				";	
		$aTeams=array();
		if ($aRows=$this->oDb->select($sql,$sUserId)) {
			foreach ($aRows as $aTeam) {
				$aTeams[]=$aTeam['team_id'];
			}
		}
		return $aTeams;
	}
	
	public function GetTeams() {
		$sql = "SELECT 
			b.team_id			 
			FROM 
				".Config::Get('db.table.teams')." as b ORDER by b.team_title";	
		$aTeams=array();
		if ($aRows=$this->oDb->select($sql)) {
			foreach ($aRows as $aTeam) {
				$aTeams[]=$aTeam['team_id'];
			}
		}
		return $aTeams;
	}
		

	/**
	 * Удаление блога из базы данных
	 *
	 * @param  int  $iBlogId
	 * @return bool	 
	 */
	public function DeleteTeam($iTeamId) {
		$sql = "
			DELETE FROM ".Config::Get('db.table.teams')." 
			WHERE team_id = ?d				
		";			
		if ($this->oDb->query($sql,$iTeamId)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Удалить пользователей блога по идентификатору блога
	 *
	 * @param  int  $iBlogId
	 * @return bool
	 */
	public function DeleteTeamUsersByTeamId($iTeamId) {
		$sql = "
			DELETE FROM ".Config::Get('db.table.team_user')." 
			WHERE blog_id = ?d
		";
		if ($this->oDb->query($sql,$iTeamId)) {
			return true;
		}
		return false;
	}
}
?>